import 'dart:convert';
import 'dart:io';

import 'package:platforms_source_gen/peoperty_parse.dart';

class JavaCreate {
  static const Map<String, String> typeMap = {
    "bool": "Boolean",
    "int": "Long",
    "double": "Double",
    "String": "String",
    "Uint8List": "byte[]",
    "Int32List": "int[]",
    "Int64List": "long[]",
    "Float64List": "double[]",
    // "List": "ArrayList",
    // "Map": "HashMap",
  };

  static void create(
      String packageName, String savePath, Map<String, List<String>> dataMap) {
    Directory androidTargetDir = new Directory(savePath);
    bool exists = androidTargetDir.existsSync();
    if (!exists) {
      androidTargetDir.createSync(recursive: true);
    }
    dataMap.forEach((key, value) {
      File javaFile = new File(savePath + "/" + key + ".java");
      List<String> imports = [
        "import java.util.ArrayList;\n",
        "import java.util.HashMap;\n"
      ];
      String allContent = "package " + packageName + ";\n";
      String propertyStr = "";
      value.forEach((element) {
        String typeStr = "";
        String name = "";
        String defaultValue = "";
        Property property = checkList(element);
        if (property != null) {
          typeStr = property.type.replaceAll("List<", "ArrayList<");
          typeMap.forEach((key, value) {
            typeStr = typeStr.replaceAll(key, value);
          });
          name = property.name;
          if (property.defaultValue1 == null ||
              property.defaultValue1.isEmpty ||
              property.defaultValue1.replaceAll(" ", "") == "[]") {
            property.defaultValue1 = "new ArrayList<>()";
          } else {
            String valueStr = "new ArrayList<>();\n\t{\n";
            List<String> a = property.defaultValue1
                .replaceAll("[", "")
                .replaceAll("]", "")
                .split(",");
            var morExt = "";
            if (typeStr.contains("Long")) {
              morExt = "L";
            }
            a.forEach((element) {
              element = element.replaceAll("\t", "").trim();
              if (element.isNotEmpty) {
                RegExp exp = new RegExp(r"\S+\([ ]*\)");
                if (exp.hasMatch(element)) {
                  String str1 = exp.firstMatch(element).group(0);
                  String str2 =
                      "import $packageName.${str1.substring(0, str1.indexOf("("))};\n";
                  if (!imports.contains(str2)) {
                    imports.add(str2);
                  }
                  element = "new " + element;
                }
                valueStr += "\t\t$name.add($element$morExt);\n";
              }
            });
            valueStr += "\t}";
            defaultValue = valueStr;
          }
        } else {
          property = checkMap(element);
          if (property != null) {
            //set type
            typeStr = property.type.replaceAll("Map<", "HashMap<");
            typeMap.forEach((key, value) {
              typeStr = typeStr.replaceAll(key, value);
            });

            //set name
            name = property.name;

            //set default value
            if (property.defaultValue1 == null ||
                property.defaultValue1.isEmpty ||
                property.defaultValue1.replaceAll(" ", "") == "{}") {
              property.defaultValue1 = "new HashMap<>()";
            } else {
              String valueStr = "new HashMap<>();\n\t{\n";
              var morExt = "";
              if (typeStr.contains("Long")) {
                morExt = "L";
              }
              List<String> a = property.defaultValue1
                  .replaceAll("{", "")
                  .replaceAll("}", "")
                  .trim()
                  .split(",");
              a.forEach((element) {
                element = element.replaceAll(" ", "");
                if (element.isNotEmpty) {
                  List<String> map = element.split(":");
                  print(map);
                  RegExp exp = new RegExp(r"\S+\([ ]*\)");
                  if (exp.hasMatch(map[0]) || exp.hasMatch(map[1])) {
                    _addObjectImport(exp, map, 0, packageName, imports);
                    _addObjectImport(exp, map, 1, packageName, imports);
                  }
                  valueStr += "\t\t$name.put(${map[0]}, ${map[1]}$morExt);\n";
                }
              });
              valueStr += "\t}";
              defaultValue = valueStr;
            }
          } else {
            property = checkNormal(element);
            typeStr = property.type;
            typeStr = typeStr
                .replaceAll("Uint8List", "byte[]")
                .replaceAll("static const", "static final");
            typeMap.forEach((key, value) {
              typeStr = typeStr.replaceAll(key, value);
            });
            name = property.name;
            defaultValue = property.defaultValue1;
            if (defaultValue != null) {
              if (typeStr == "Long") {
                defaultValue += "L";
              } else if (typeStr == "byte[]") {
                defaultValue = defaultValue
                    .replaceAll("Uint8List", "new byte[")
                    .replaceAll("(", "")
                    .replaceAll(")", "");
                defaultValue += "]";
              } else {
                typeMap.forEach((key, value) {
                  defaultValue = defaultValue
                      .replaceAll(key, "new " + value)
                      .replaceAll("]", "")
                      .replaceAll("(", "")
                      .replaceAll(")", "");
                  if (defaultValue.contains("new ")) {
                    defaultValue += "]";
                  }
                });
              }
            }
          }
        }
        String tmpDefaultValue = "";
        if (defaultValue == null || defaultValue.isEmpty) {
          tmpDefaultValue = "";
        } else {
          tmpDefaultValue = "= $defaultValue";
        }
        propertyStr += "\tpublic $typeStr $name $tmpDefaultValue;\n";
      });
      String importStr = imports
          .toString()
          .replaceAll("[", "")
          .replaceAll("]", "")
          .replaceAll(",", "");

      allContent += "${importStr}public class $key {\n$propertyStr}";
      javaFile.writeAsString(allContent);
    });
  }

  static void _addObjectImport(RegExp exp, List<String> map, int index,
      String packageName, List<String> imports) {
    String str1 = "";
    try {
      str1 = exp.firstMatch(map[index]).group(0);
      map[index] = "new " + map[index];
    } catch (e) {}
    String str2 =
        "import $packageName.${str1.substring(0, str1.indexOf("("))};\n";
    if (!imports.contains(str2)) {
      imports.add(str2);
    }
  }
}
